// Cellule class 3D
// Creation: 25/11/2008
// Last modification: 20/01/2009
package maillage;

import java.util.Vector;
import java.lang.Math.*;


/** 
 * Classe d'une cellule de voronoi
 */

public class Cell_voronoi extends Cellule {

  private Vector<Cell_voronoi> Neighbours;
  private Vector<Point> voronoi;

  /**
   * Constructeur d'une cellule de Voronoi à partir d'un point q 
   * @param Q Point de la cellule
   */
  public Cell_voronoi (Point q)
  {
    super(q);
    Neighbours = new Vector<Cell_voronoi>();
    voronoi = new Vector<Point>();
  }

  /** 
   * Retourne la liste de voisins d'une cellule
   * @return liste de voisins
   */
  public Vector getNeighbours()
  {
    return Neighbours;
  }
    
  /** 
   * Returns the number of neighbours of the cell
   * @return Number of neighbours of the cell
   */
  public int getNumberNeighbours()
  {
    return Neighbours.size();
  }
      
  /** 
   * Checks if a cell is a neighbour of the considered cell
   * @param cell Test cell
   */
  public boolean checkNeighbour (Cell_voronoi cell) {
    if (Neighbours.contains(cell))
    {
      return true;
    }
    else {
      return false;
    }
  }

  /** 
   * Adds a neighbour to the cell
   * @param cell Cell to add
   * @param p Associated probability
   */
  public void addVoisins (Cell_voronoi cell)
  {
    if (this.checkNeighbour(cell) == false) 
    {
      Neighbours.add(cell);
    }
  }
   
  /** 
   * Removes a neighbour to the cell 
   * @param cell Cell to remove
   */
  public void removeNeighbour (Cell_voronoi cell)
  {
    if (this.checkNeighbour(cell) == true)
    {
      int i = Neighbours.indexOf(cell);
      Neighbours.remove(i);
    }
  }
    
  /**
   * Generates the cell list of a spherical grid from a location structure
   * @param grid the concerned spherical grid
   * @param graph the location structure used to build the cell list
   * @return cellList Vector containing all the generated cells 
   */
  public static Vector cellGeneration(LocationStructure graph)
  {
    Vector<Cell_voronoi> cellList = new Vector<Cell_voronoi>();
    Vector pointList = new Vector();
    Vector leafList = graph.getLeafList();

    // On teste tout les triangles et on associe pour chaque sommet une cellule (sans redondances)
    for(int i=0;i<leafList.size();i++)
    {
      Triangle tr = ((GraphNode)(leafList.get(i))).getTriangle();

      Point P1 = tr.getS1();
      Point P2 = tr.getS2();
      Point P3 = tr.getS3();

      Cell_voronoi cell1 = null;
      Cell_voronoi cell2 = null;
      Cell_voronoi cell3 = null;

      // Test sur le premier sommet
      int test = 0;
      for (int j=0; j <cellList.size();j++)
      {
        cell1 = (Cell_voronoi)cellList.get(j);
	      if (P1 == cell1.getPoint())
        {
          test =1;
          break;
        }
      }
      if (test == 0)
      {
        cell1 = new Cell_voronoi(P1);
        cellList.add(cell1);
      }

      // Test sur le second sommet
      test=0;
      for (int j=0; j <cellList.size();j++)
      {
        cell2 = (Cell_voronoi)cellList.get(j);
	if (P2 == cell2.getPoint())
        {
          test =1;
	  break;
	}
      }
      if (test == 0)
      {
        cell2 = new Cell_voronoi(P2);
        cellList.add(cell2);
      }

      // Test sur le troisieme sommet
      test=0;
      for (int j=0; j <cellList.size();j++)
      {
        cell3 = (Cell_voronoi)cellList.get(j);
	if (P3 == cell3.getPoint())
        {
          test =1;
          break;
        }
      }
      if (test == 0)
      {
        cell3 = new Cell_voronoi(P3);
        cellList.add(cell3);
      }
      
      // On a une liste avec toutes les cellules : cellList
      // A chaque cellule on ajoute ses voisins (ie les autres sommets du triangle associés au noeud)
 
      cell1.addVoisins(cell2);
      cell1.addVoisins(cell3);

      cell2.addVoisins(cell1);
      cell2.addVoisins(cell3);

      cell3.addVoisins(cell1);
      cell3.addVoisins(cell2);    
      }
 
      // Pour chaque cellule on génère le diagramme de Voronoi
      for(int i=0;i<cellList.size();i++)
      {
        Cell_voronoi c = (Cell_voronoi)cellList.get(i);
        c.generateVoronoi();
      }
      return cellList;
    }

  /** 
   * Generates the Voronoi vertices of the Cell
   */
  public void generateVoronoi()
  {
    int n = this.getNumberNeighbours();
    Vector N = this.getNeighbours();	
    Vector vorList = new Vector(); 
    
    Point O = this.getPoint(); 
	
    // Les voisins doivent etre classes !
    for (int i=0; i<n; i++)
    {
      Cell_voronoi C1;
      Cell_voronoi C2;
      Cell_voronoi C3;
      if (i == n-1)
      {
        C1 = (Cell_voronoi)((Vector)N).get(i);
        C2 = (Cell_voronoi)((Vector)N).get(0);
        C3 = (Cell_voronoi)((Vector)N).get(1);
      }
      else if (i == n-2)
      {	
        C1 = (Cell_voronoi)((Vector)N).get(i);
        C2 = (Cell_voronoi)((Vector)N).get(i+1);
        C3 = (Cell_voronoi)((Vector)N).get(0);
      }
      else
      {
        C1 = (Cell_voronoi)((Vector)N).get(i);
        C2 = (Cell_voronoi)((Vector)N).get(i+1);
        C3 = (Cell_voronoi)((Vector)N).get(i+2);
      }
    
      Point A = C1.getPoint();
      Point B = C2.getPoint();
      Point C = C3.getPoint();

      //On cherche le point d'intersection entre les points
      Point I = O.intersec(A,B);
      //Point J = O.intersec(B,C);   
      vorList.add(I);  


    } 
    this.voronoi =  vorList;
  }

    /**
    * Getter for the "voronoi" attribute
    */
    public Vector getVoronoi(){
	return voronoi;
    }

}   

