// Classe Cellule
// Creation: 14/10/2012
// Last modification: 15/10/2012

package maillage;

import java.util.Vector;
//import java.awt.geom.Point2D;

/**
 * Classe Cellule
 * Element elementaire d'une Grille
 */

public class Cellule
  {

    private Point pts;
    private Vector<Cellule> voisins;
    private boolean etat;

    /**
     * Constructeur d'une cellule sans aucun voisin a partir d'un point Q
     * @param q Position de la nouvelle cellule
     */

    public Cellule (Point q)
    {
	    pts = q;
	    voisins = new Vector<Cellule>();
	    etat = false;
    }

    /**
     * Constructeur d'une cellule sans aucun voisin a partir d'un point q
     * @param q Position de la nouvelle cellule
     * @param v Voisin de la nouvelle cellule
     */

    public Cellule (Point q,Vector<Cellule> v)
    {
	    pts = q;
	    voisins = v;
	    etat = false;
    }

     /**
     * Constructeur d'une cellule sans aucun voisin a partir d'un point q
     * @param q Position de la nouvelle cellule
     * @param v Voisin de la nouvelle cellule
     * @param e Etat de la nouvelle cellule
     */

    public Cellule (Point q,Vector<Cellule> v,boolean e)
    {
	    pts = q;
	    voisins = v;
	    etat = e;
    }

     /**
     * Constructeur d'une cellule sans aucun voisin a partir d'un point q
     * @param q Position de la nouvelle cellule
     * @param e Etat de la nouvelle cellule
     */

    public Cellule (Point q,boolean e)
    {
	    pts = q;
	    voisins = new Vector<Cellule>();
	    etat = e;
    }

    /**
     * Retourne le point de la cellule
     * @return pts Point de la cellule
     */
    public Point getPoint()
    {
	    return pts;
    }

    /**
     * Retourne le statut de la cellule
     * @return etat Statut de la cellule: true ou false
     */
    public boolean statut()
    {
	    return etat;
    }

    /**
     * Retourne les voisins de la cellule
     * @return voisins Les voisins de la cellule
     */
    public Vector<Cellule> getVoisins()
    {
	    return voisins;
    }

    /**
     * Verifie si les cellules sont voisines
     * @param c Cellule a tester
     */
    public boolean estVoisins (Cellule c)
    {
	return voisins.contains(c);
    }

    /**
     * Ajoute un voisin a une cellule
     * @param c Cellule a ajouter
     */
    public void ajouterVoisin (Cellule c)
    {
      if (!this.estVoisins(c))
      {
	      voisins.addElement(c);
	    }
    }

    /**
     * Supprimer un voisin d'une cellule
     * @param c Cellule a supprimer
     */
    public void  supprimerVoisin (Cellule c)
    {
	    if (this.estVoisins(c))
	    {
	      int i = voisins.indexOf(c);
	      voisins.removeElementAt(i);
	    }
    }

    /**
     * R&eacute;duit la taille du tableau voisin au maximum.
     */
    public void  trim ()
    {

      voisins.trimToSize();

    }

    /**
     * Supprimer tous les voisins d'une cellule
     */
    public void  supprimerAllVoisins ()
    {
      voisins.removeAllElements();
    }


    /**
     * Returne la distance euclidienne entre deux cellules
     * @param c Cellule
     * @return Distance entre les deux cellules
     */
    public double distance(Cellule c)
    {
	    Point q = c.getPoint();

	    return pts.distance(q);
    }

    /**
     * Returne le nombre de voisins d'une cellule
     * @return nombre de voisins d'une cellule
     */
    public int nombreVoisins()
    {
	    return voisins.size();
    }



    /**
     * Change le statut de la cellule
     */
    public void changementStatut ()
    {
	    this.etat = !etat;
    }

    /**
     * Remplace l'etat de la cellule par e
     */
    public void setStatut (boolean e)
    {
	    this.etat = e;
    }

  }

