package maillage;
import java.util.Vector;
//import java.awt.geom.Point2D;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.image.*;
import java.awt.Color;

/**
 * Classe m&egrave;re pour les grilles
 */
public abstract class Grille
{

  protected Vector<Cellule> listeCellules;

  protected int nlig;
  protected int ncol;

  /**
  * Recupere le nombre de lignes
  * @return Le nombre de lignes
  */

  public int getNombreLigne()
  {
    return nlig;
  }

  /**
  * Recupere le nombre de colonnes
  * @return Le nombre de colonnes
  */

  public int getNombreColonne()
  {
    return ncol;
  }

  /**
  * Recupere le nombre de cellule
  * @return Le nombre de cellule
  */

  public int getNombreCellule()
  {
    return listeCellules.size();
  }

  /**
  * Recupere le vecteur de cellule de la grille
  * @return Le vecteur de cellule de la grille
  */

  public Vector<Cellule> getListeCellules()
  {
    return listeCellules;
  }

  /**
  * Effectue la conversion d'une grille en tableau de r&eacute;els en double pr&eacute;cision.
  * Il est conseill&eacute; d'utiliser cette m&eacute;thode uniquement dans le cas o&ugrave; g&eacute;n&eacute;rer 
  * un nouveau tableau est n&eacute;cessaire.
  * Attention&nbsp;! les dimensions du tableau doivent &ecirc;tre sup&eacute;rieures ou &eacute;gales &agrave; celles de la grille multipli&eacute;es par le zoom.
  * @param n Entier correspondant au nombre de lignes du tableau de sortie.
  * @param m Entier correspondant au nombre de colonnes du tableau de sortie.
  * @param gamma Entier permettant d'effectuer un zoom.
  * @return Un tableau de r&eacute;els en double pr&eacute;cision correspondant &agrave; la grille.
  */

  abstract public double[][] conversion(int n, int m,int gamma);

  /**
  * Effectue la conversion d'une grille en tableau de r&eacute;els en double pr&eacute;cision.
  * Attention&nbsp;! les dimensions du tableau doivent &ecirc;tre sup&eacute;rieures ou &eacute;gales &agrave; celles de la grille multipli&eacute;es par le zoom.
  * @param image Un tableau de r&eacute;els en double pr&eacute;cision correspondant &agrave; la grille.
  * @param gamma Entier permettant d'effectuer un zoom.
  */

  abstract public void conversion(double[][] image,int gamma);

  /**
  * Effectue la conversion d'une grille en un objet graphique.
  * @param graph Objet graphique correspondant &agrave; la grille.
  * @param gamma Entier permettant d'effectuer un zoom.
  */

  abstract public void conversionGraphics(Graphics graph,int gamma,Color c1,Color c2);

  /**
  * Retourne l'index de la cellule c
  * @param c Cellule consideree
  * @return index de la cellule
  */

  abstract public int indexOf(Cellule c);

  /**
  * Recupere la cellule de d'index i
  * @param i Entier correspondant a l'index de la cellule
  * @return Cellule d'index i
  */

  public Cellule getCellule(int i)
  {
    i=i%(listeCellules.size());
    return listeCellules.get(i);
  }

  /*
  * Translate les voisins d'une cellule vers une autre
  * @param a Entier index de la cellule dont les voisins sont connus
  * @param c Entier index de la cellule dont on veut connaitre les voisins
  */

  //abstract public void translateVoisins(int a,int c);


  /**
  * Ajoute les voisins de la cellule d'index j present dans un rayon 'rayon'
  * @param j Entier index de la cellule
  * @param rayon Entier correspondant au rayon de recherche des voisins
  */

  abstract public void rechercheVoisins(int j,int rayon);

  /**
  * Ajoute les voisins d'une cellule, dont l'index est sp&eacute;cifi&eacute;, pr&eacute;sents dans un certain rayon.
  * @param i Entier correspondant &agrave; l'index de la cellule.
  * @param e Booléen permettant l'utilisation ou non des conditions periodiques
  * @param rayon Entier correspondant au rayon de recherche des voisins.
  */

  abstract public void rechercheVoisins(int j, boolean e, int rayon);

    /**
     * Returne la plus petite distance euclidienne entre deux cellules
     * @param c1 Cellule
     * @param c2 Cellule
     * @return Distance entre les deux cellules
     */
    public double distance(Cellule c1,Cellule c2)
    {
      Point q1 = c1.getPoint();
      Point q2 = c2.getPoint();
      double d = q1.distance(q2);
      double x1=q1.getX();
      double y1=q1.getY();
      double x2=q2.getX();
      double y2=q2.getY();

      for (int i=-1;i<2;i++)
      {
       for (int j=-1;j<2;j++)
       {
         d=Math.min(d,q1.distance(x2+i*nlig,y2+j*ncol));
       }
      }

      return d;
    }

    /**
     * Returne la plus petite distance euclidienne entre deux cellules
     * @param l index de la premiere cellule
     * @param k index de la deuxieme cellule
     * @return Distance entre les deux cellules
     */
    public double distance(int l,int k)
    {
      Cellule c1 = listeCellules.get(l);
      Cellule c2 = listeCellules.get(k);
      return this.distance(c1,c2);
    }


  }

