package maillage;

import java.util.Vector;
//import java.awt.geom.Point2D;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.image.*;
import java.awt.Color;
/**
 * Classe heritee de la classe grille
 * Classe dans le cas d'une grille rectangulaire a pavage carree
 */



public class GrilleCarree extends Grille
{

  /**
  * Constructeur d'une grille carree
  * @param taille Entier correspondant a la taille des cotes
  */

  public GrilleCarree(int taille)
  {
    nlig = taille;
    ncol = nlig;
    listeCellules = new Vector<Cellule>();
    for (int i=0;i<nlig;i++)
    {
      for (int j=0;j<ncol;j++)
      {
        Point p = new Point(i,j);
        Cellule c = new Cellule(p);
        listeCellules.add(c);
        double r = Math.random();
        if (r<0.5)
        {
          (listeCellules.get(i*ncol+j)).changementStatut();
        }
      }
    }
  }


  /**
  * Constructeur d'une grille carree
  * @param taille Entier correspondant a la taille des cotes
  * @param e Etat de toutes les cellules
  */

  public GrilleCarree(int taille,boolean e)
  {
    nlig = taille;
    ncol = nlig;
    listeCellules = new Vector<Cellule>();
    for (int i=0;i<nlig;i++)
    {
      for (int j=0;j<ncol;j++)
      {
        Point p = new Point(i,j);
        Cellule c = new Cellule(p,e);
        listeCellules.add(c);
      }
    }
  }

  /**
  * Constructeur d'une grille carree
  * @param nl Entier correspondant a la taille d'un des cotes
  * @param nc Entier correspondant a la taille d'un des cotes
  * @param e Etat de toutes les cellules
  */

  public GrilleCarree(int nl,int nc,boolean e)
  {
    nlig = nl;
    ncol = nc;
    listeCellules = new Vector<Cellule>();
    for (int i=0;i<nlig;i++)
    {
      for (int j=0;j<ncol;j++)
      {
        Point p = new Point((double)i,(double)j);
        Cellule c = new Cellule(p,e);
        listeCellules.add(c);
      }
    }
  }

  /**
  * Constructeur d'une grille rectangulaire
  * @param nl Entier correspondant a la taille d'un des cotes
  * @param nc Entier correspondant a la taille d'un des cotes
  */

  public GrilleCarree(int nl,int nc)
  {
    nlig = nl;
    ncol = nc;
    listeCellules = new Vector<Cellule>();
    for (int i=0;i<nlig;i++)
    {
      for (int j=0;j<ncol;j++)
      {
        Point p = new Point(i,j);
        Cellule c = new Cellule(p);
        listeCellules.add(c);
        double r = Math.random();
        if (r<0.5)
        {
          (listeCellules.get(i*ncol+j)).changementStatut();
        }
      }
    }
  }

  /**
  * Recupere la cellule de coordonnees (i,j)
  * @param i Entier correspondant a l'une des coordonnees de la cellule
  * @param j Entier correspondant a l'une des coordonnees de la cellule
  * @return Cellule de coordonnees (i,j)
  */

  public Cellule getCellule(int i,int j)
  {
    i=i%nlig;
    j=j%ncol;
    return listeCellules.get(i*ncol + j);
  }

  /**
  * Effectue la conversion d'une grille en tableau de reels en double precision
  * @param n Entier correspondant au nombre de lignes de l'image a afficher
  * @param m Entier correspondant au nombre de colonnes de l'image a afficher
  * @param gamma Entier correspondant au grandissement souhaite
  * @return Un tableau de reels en double precision correspondant a l'image
  */

  public double[][] conversion(int n,int m,int gamma)
  {
    double[][] image = new double[n][m];
    double valeurPixel = 255.0;

    for (int k=0;k<nlig ;k++)
    {
      for (int l=0;l<ncol ;l++)
      {
        int i=gamma*k;
        int j=gamma*l;

        image[i][j]=255.0;

        if (!(listeCellules.get(k*ncol+l).statut()))
        {
          image[i][j]-=valeurPixel;
        }

        for (int ii=0;ii<gamma ;ii++)
        {
          for (int jj=0;jj<gamma ;jj++)
          {
            image[i+ii][j+jj]=image[i][j];
          }
        }

      }

    }
    return image;
  }

  public void conversion(double[][] image,int gamma)
  {
    double valeurPixel = 255.0;

    for (int k=0;k<nlig ;k++)
    {
      for (int l=0;l<ncol ;l++)
      {
        int i=gamma*k;
        int j=gamma*l;

        image[i][j]=255.0;

        if (!(listeCellules.get(k*ncol+l).statut()))
        {
          image[i][j]-=valeurPixel;
        }

        for (int ii=0;ii<gamma ;ii++)
        {
          for (int jj=0;jj<gamma ;jj++)
          {
            image[i+ii][j+jj]=image[i][j];
          }
        }

      }

    }
  }

   public void conversionGraphics(Graphics graph,int gamma,Color c1,Color c2)
     {
       for (int k=0;k<nlig ;k++)
       {
         for (int l=0;l<ncol ;l++)
         {

           int m=gamma*k;
           int j=gamma*l;
           int[] vectorX = new int[4];
           int[] vectorY = new int[4];


           vectorX[0]=m;
           vectorY[0]=j;

           vectorX[3]=m+gamma;
           vectorY[3]=j;

           vectorX[1]=m;
           vectorY[1]=j+gamma;

           vectorX[2]=m+gamma;
           vectorY[2]=j+gamma;


           if (!(listeCellules.get(k*ncol+l).statut()))
           {
             graph.setColor(c1);
           }
           else
           {
             graph.setColor(c2);
           }

           graph.fillPolygon(vectorX,vectorY,4);

         }

       }
     }

  /**
  * Ajoute les voisins d'une cellule, dont les coordonn&eacute;es sont sp&eacute;cifi&eacute;es, pr&eacute;sents dans un certain rayon.
  * @param i Entier correspondant &agrave; l'une des coordon&eacute;es de la cellule.
  * @param j Entier correspondant &agrave; l'une des coordon&eacute;es de la cellule.
  * @param rayon Entier correspondant au rayon de recherche des voisins.
  */

  public void rechercheVoisins(int i,int j,int rayon)
  {
    i=i%nlig;
    j=j%ncol;
    Cellule c = (listeCellules.get(i*ncol+j));



    for (int ii=i-rayon;ii<i+rayon+1;ii++)
    {
      /*int root = (int) Math.ceil(Math.sqrt(rayon*rayon-(ii-i)*(ii-i)));
      root++;*/
      for (int jj=j-rayon;jj<j+rayon+1;jj++)
      //for (int jj=j-root;jj<j+root+1;jj++)
      {
        if (ii != i || jj != j)
        {
          Cellule cTemp = (listeCellules.get(((ii+nlig)%nlig)*ncol+((jj+ncol)%ncol)));
          double d = this.distance(c,cTemp);
          if (d<=rayon)
          {
	          c.ajouterVoisin(cTemp);
          }
        }
      }
    }

    c.trim();

  }

    /**
    * Ajoute les voisins d'une cellule, dont les coordonn&eacute;es sont sp&eacute;cifi&eacute;es, pr&eacute;sents dans un certain rayon.
    * @param i Entier correspondant &agrave; l'une des coordon&eacute;es de la cellule.
    * @param j Entier correspondant &agrave; l'une des coordon&eacute;es de la cellule.
    * @param e Booléen permettant l'utilisation ou non des conditions periodiques
    * @param rayon Entier correspondant au rayon de recherche des voisins.
    */

    public void rechercheVoisins(int i,int j,boolean e,int rayon)
    {
      i=i%nlig;
      j=j%ncol;
      Cellule c = (listeCellules.get(i*ncol+j));



      for (int ii=i-rayon;ii<i+rayon+1;ii++)
      {
        /*int root = (int) Math.ceil(Math.sqrt(rayon*rayon-(ii-i)*(ii-i)));
        root++;*/
        for (int jj=j-rayon;jj<j+rayon+1;jj++)
        //for (int jj=j-root;jj<j+root+1;jj++)
        {
          if (ii != i || jj != j)
          {
            double d;
            Cellule cTemp = (listeCellules.get(((ii+nlig)%nlig)*ncol+((jj+ncol)%ncol)));
            if (e)
            {
              d = this.distance(c,cTemp);
            }
            else
            {
              d = c.distance(cTemp);
            }
            if (d<=rayon)
            {
              c.ajouterVoisin(cTemp);
            }
          }
        }
      }

      c.trim();

    }

  /*
  * Translate les voisins d'une cellule vers une autre! Bug!!!!
  * @param a Entier index de la cellule dont les voisins sont connus
  * @param b Entier index de la cellule dont les voisins sont connus
  * @param c Entier index de la cellule dont on veut connaitre les voisins
  * @param d Entier index de la cellule dont on veut connaitre les voisins
  */

  /*public void translateVoisins(int a,int b,int c,int d)
  {
    a=a%nlig;
    b=b%ncol;
    c=c%nlig;
    d=d%ncol;

    if (a!=c && b!=d)
    {
      Cellule c1 = (listeCellules.get(a*ncol+b));
      Cellule c2 = (listeCellules.get(c*ncol+d));

      Vector<Cellule> c1Voisins = c1.getVoisins();
      int n=c1Voisins.size();
      for (int k=0;k<n;k++)
      {
        Cellule cTemp = c1Voisins.get(k);
        int i1 = (int)cTemp.getPoint().getX();
        int j1 = (int)cTemp.getPoint().getY();
        int i2 = (i1 + a - c + nlig)%nlig;
        int j2 = (j1 + b - d + ncol)%ncol;
        Cellule cTemp2 = listeCellules.get(i2*ncol+j2);
        c2.ajouterVoisin(cTemp2);
      }

    }
  }*/

  /**
  * Retourne l'index de la cellule c
  * @param c Cellule consideree
  * @return index de la cellule
  */

  public int indexOf(Cellule c)
  {
    Point p = c.getPoint();
    int i = (int)p.getX();
    int j = (int)p.getY();
    return i*ncol+j;
  }

  /**
  * Ajoute les voisins d'une cellule, dont l'index est sp&eacute;cifi&eacute;, pr&eacute;sents dans un certain rayon.
  * @param i Entier correspondant &agrave; l'index de la cellule.
  * @param rayon Entier correspondant au rayon de recherche des voisins.
  */
  
  public void rechercheVoisins(int i,int rayon)
  {
    this.rechercheVoisins(i/ncol,i%ncol,rayon);
  }

  /**
  * Ajoute les voisins d'une cellule, dont l'index est sp&eacute;cifi&eacute;, pr&eacute;sents dans un certain rayon.
  * @param i Entier correspondant &agrave; l'index de la cellule.
  * @param e Booléen permettant l'utilisation ou non des conditions periodiques
  * @param rayon Entier correspondant au rayon de recherche des voisins.
  */
  
  public void rechercheVoisins(int i, boolean e, int rayon)
  {
    this.rechercheVoisins(i/ncol,i%ncol,e,rayon);
  }

}

