// Classe GrilleHexa
// Creation: 14/10/2012
// Last modification: 14/11/2012

package maillage;

import java.util.Vector;
//import java.awt.geom.Point2D;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.image.*;
import java.awt.Color;

/**
* Classe permettant la création d'une grille rectangulaire &agrave; pavage hexagonal.
* Pour obtenir des conditions aux limites p&eacute;riodiques, on impose un nombre de lignes paire.
* <img src="http://images.math.cnrs.fr/IMG/gif/graphe_hexagonal-2.gif" width="200">
*/

public class GrilleHexa extends Grille
{

  /**
  * Constructeur d'une grille &agrave; pavage hexagonale dont les cellules sont g&eacute;n&eacute;r&eacute;es al&eacute;atoirement.
  * Si le nombre de lignes est impaire, il sera remplac&eacute; par l'entier paire inf&eacute;rieur le plus proche. 
  * @param taille Entier correspondant au nombre de lignes et de colonnes.
  */

  public GrilleHexa(int taille)
  {
    nlig = taille-taille%2;
    ncol = taille;
    listeCellules = new Vector<Cellule>();
    for (int i=0;i<nlig;i++)
    {
      for (int j=0;j<ncol;j++)
      {
        Point p = new Point((Math.sqrt(3.0)*0.5*i),0.5*(2*j+i%2));
        Cellule c = new Cellule(p);
        listeCellules.add(c);
        double r = Math.random();
        if (r<0.5)
        {
          (listeCellules.get(i*ncol+j)).changementStatut();
        }
      }
    }
  }

  /**
  * Constructeur d'une grille &agrave; pavage hexagonale dont les cellules sont g&eacute;n&eacute;r&eacute;es al&eacute;atoirement.
  * Si le nombre de lignes est impaire, il sera remplac&eacute; par l'entier paire inf&eacute;rieur le plus proche. 
  * @param nombreLignes Entier correspondant au nombre de lignes.
  * @param nombreColonnes Entier correspondant au nombre de colonnes.
  */

  public GrilleHexa(int nombreLignes,int nombreColonnes)
  {
    nlig = nombreLignes-nombreLignes%2;
    ncol = nombreColonnes;
    listeCellules = new Vector<Cellule>();
    for (int i=0;i<nlig;i++)
    {
      for (int j=0;j<ncol;j++)
      {
        Point p = new Point((Math.sqrt(3.0)*0.5*i),0.5*(2*j+i%2));
        Cellule c = new Cellule(p);
        listeCellules.add(c);
        double r = Math.random();
        if (r<0.5)
        {
          (listeCellules.get(i*ncol+j)).changementStatut();
        }
      }
    }
  }
  
  /**
  * Constructeur d'une grille &agrave; pavage hexagonale dont les cellules sont toutes dans l'&eacute;tat sp&eacute;cifi&eacute;.
  * Si le nombre de lignes est impaire, il sera remplac&eacute; par l'entier paire inf&eacute;rieur le plus proche. 
  * @param taille Entier correspondant au nombre de lignes et de colonnes.
  * @param e bool&eacute;en correspondant &agrave; l'&eacute;tat sp&eacute;cifi&eacute; pour toutes les cellules de la grille.
  */

  public GrilleHexa(int taille,boolean e)
  {
    nlig = taille-taille%2;
    ncol = taille;
    listeCellules = new Vector<Cellule>();
    for (int i=0;i<nlig;i++)
    {
      for (int j=0;j<ncol;j++)
      {
        Point p = new Point((Math.sqrt(3.0)*0.5*i),0.5*(2*j+i%2));
        Cellule c = new Cellule(p);
        c.setStatut(e);
        listeCellules.add(c);
      }
    }
  }

  /**
  * Constructeur d'une grille &agrave; pavage hexagonale dont les cellules sont toutes dans l'&eacute;tat sp&eacute;cifi&eacute;.
  * Si le nombre de lignes est impaire, il sera remplac&eacute; par l'entier paire inf&eacute;rieur le plus proche. 
  * @param nombreLignes Entier correspondant au nombre de lignes.
  * @param nombreColonnes Entier correspondant au nombre de colonnes.
  * @param e bool&eacute;en correspondant &agrave; l'&eacute;tat sp&eacute;cifi&eacute; pour toutes les cellules de la grille.
  */

  public GrilleHexa(int nombreLignes,int nombreColonnes,boolean e)
  {
    nlig = nombreLignes-nombreLignes%2;
    ncol = nombreColonnes;
    listeCellules = new Vector<Cellule>();
    for (int i=0;i<nlig;i++)
    {
      for (int j=0;j<ncol;j++)
      {
        Point p = new Point((Math.sqrt(3.0)*0.5*i),0.5*(2*j+i%2));
      //System.out.println(p);
        Cellule c = new Cellule(p);
        c.setStatut(e);
        listeCellules.add(c);
      }
    }
  }

  /**
  * Retourne l'index d'une cellule.
  * @param c Cellule consider&eacute;e.
  * @return Index de la cellule.
  */
  
  public int indexOf(Cellule c)
  {
    Point p = c.getPoint();
    int i = (int)(p.getX()*2.0/Math.sqrt(3.0));
    int j = ((int)(2.0*p.getY()))/2;
    return i*ncol+j;
  }
  
  /**
  * Effectue la conversion d'une grille en tableau de r&eacute;els en double pr&eacute;cision.
  * Il est conseill&eacute; d'utiliser cette m&eacute;thode uniquement dans le cas o&ugrave; g&eacute;n&eacute;rer 
  * un nouveau tableau est n&eacute;cessaire.
  * Attention&nbsp;! les dimensions du tableau doivent &ecirc;tre sup&eacute;rieures ou &eacute;gales &agrave; celles de la grille multipli&eacute;es par le zoom.
  * @param n Entier correspondant au nombre de lignes du tableau de sortie.
  * @param m Entier correspondant au nombre de colonnes du tableau de sortie.
  * @param gamma Entier permettant d'effectuer un zoom.
  * @return Un tableau de r&eacute;els en double pr&eacute;cision correspondant &agrave; la grille.
  */
  
  public double[][] conversion(int n,int m,int gamma)
  {
   double[][] image = new double[n][m];
   double valeurPixel = 255.0;
  
   for (int k=0;k<nlig ;k++)
   {
     for (int l=0;l<ncol ;l++)
     {
       int i=gamma*k;
       int j=gamma*l;
  
       image[i][j]=255.0;
  
       if (!(listeCellules.get(k*ncol+l).statut()))
       {
         image[i][j]-=valeurPixel;
       }
  
       for (int ii=0;ii<gamma ;ii++)
       {
         for (int jj=0;jj<gamma ;jj++)
         {
           image[i+ii][j+jj]=image[i][j];
         }
       }
  
     }
  
   }
   return image;
  }

  /**
  * Effectue la conversion d'une grille en tableau de r&eacute;els en double pr&eacute;cision.
  * Attention&nbsp;! les dimensions du tableau doivent &ecirc;tre sup&eacute;rieures ou &eacute;gales &agrave; celles de la grille multipli&eacute;es par le zoom.
  * @param image Un tableau de r&eacute;els en double pr&eacute;cision correspondant &agrave; la grille.
  * @param gamma Entier permettant d'effectuer un zoom.
  */
  
  public void conversion(double[][] image,int gamma)
  {
  
    double valeurPixel = 255.0;
  
    for (int k=0;k<nlig ;k++)
    {
      for (int l=0;l<ncol ;l++)
      {
        int i=gamma*k;
        int j=gamma*l;
  
        image[i][j]=255.0;
  
        if (!(listeCellules.get(k*ncol+l).statut()))
        {
          image[i][j]-=valeurPixel;
        }
  
        for (int ii=0;ii<gamma ;ii++)
        {
          for (int jj=0;jj<gamma ;jj++)
          {
            image[i+ii][j+jj]=image[i][j];
          }
        }
  
      }
  
    }
  }

  /**
  * Effectue la conversion d'une grille en un objet graphique.
  * @param graph Objet graphique correspondant &agrave; la grille.
  * @param gamma Entier permettant d'effectuer un zoom.
  */
  
  public void conversionGraphics(Graphics graph,int gamma,Color c1,Color c2)
  {
    //graph.setColor(Color.red);
    //graph.fillRect(0,0,(int)(Math.sqrt(3.0)/2.0*nlig*gamma),ncol*gamma);
    for (int k=0;k<nlig ;k++)
    {
      for (int l=0;l<ncol ;l++)
      {
        int[] vectorX = new int[6];
        int[] vectorY = new int[6];
        Point p = (listeCellules.get(k*ncol+l)).getPoint();
  
        for (int i=0;i<6 ;i++)
        {
          double x = (Math.cos((60.0*i)*Math.PI/180.0)/Math.sqrt(3.0)+p.getX()+0.5);
          double y = (Math.sin((60.0*i)*Math.PI/180.0)/Math.sqrt(3.0)+p.getY()+0.5);
  
  
          vectorX[i]=(int)(gamma*x);
          vectorY[i]=(int)(gamma*y);
  
        }
  
        if (!(listeCellules.get(k*ncol+l).statut()))
        {
          graph.setColor(c1);
        }
        else
        {
          graph.setColor(c2);
        }
        graph.fillPolygon(vectorX,vectorY,6);
      }
  
    }
  }
  
  /**
  * Ajoute les voisins d'une cellule, dont les coordonn&eacute;es sont sp&eacute;cifi&eacute;es, pr&eacute;sents dans un certain rayon.
  * @param i Entier correspondant &agrave; l'une des coordon&eacute;es de la cellule.
  * @param j Entier correspondant &agrave; l'une des coordon&eacute;es de la cellule.
  * @param rayon Entier correspondant au rayon de recherche des voisins.
  */
  
  public void rechercheVoisins(int i,int j,int rayon)
  {
    
    i=i%nlig;
    j=j%ncol;

    Cellule c = (listeCellules.get(i*ncol+j));
    int rHexa=(int)(2.0*rayon/Math.sqrt(3))+1;
    for (int ii=i-rHexa;ii<i+rHexa+1;ii++)
    {
      for (int jj=j-rayon;jj<j+rayon+1;jj++)
      {
        if (ii != i || jj != j)
        {
          Cellule cTemp = (listeCellules.get(((ii+nlig)%nlig)*ncol+((jj+ncol)%ncol)));
          double d = this.distance(c,cTemp);
          if (d<=rayon+10e-6)
          {
           c.ajouterVoisin(cTemp);
          }
        }
      }
    }

    c.trim();

  }
  
  /**
  * Ajoute les voisins d'une cellule, dont l'index est sp&eacute;cifi&eacute;, pr&eacute;sents dans un certain rayon.
  * @param i Entier correspondant &agrave; l'index de la cellule.
  * @param rayon Entier correspondant au rayon de recherche des voisins.
  */
  
  public void rechercheVoisins(int i,int rayon)
  {
    this.rechercheVoisins(i/ncol,i%ncol,rayon);
  }
  
  /**
  * Retourne la plus petite distance euclidienne entre deux cellules.
  * @param c1 Premi&egrave;re cellule. 
  * @param c2 Deuxi&egrave;me cellule.
  * @return Distance entre les deux cellules.
  */

  public double distance(Cellule c1,Cellule c2)
  {
    Point q1 = c1.getPoint();
    Point q2 = c2.getPoint();
    double d = q1.distance(q2);
    double x1=q1.getX();
    double y1=q1.getY();
    double x2=q2.getX();
    double y2=q2.getY();

    for (int i=-1;i<2;i++)
    {
      for (int j=-1;j<2;j++)
      {
        d=Math.min(d,q1.distance(x2+i*nlig*Math.sqrt(3.0)*0.5,y2+j*ncol));
      }
    }

    return d;
  }

  /**
  * Ajoute les voisins d'une cellule, dont l'index est sp&eacute;cifi&eacute;, pr&eacute;sents dans un certain rayon.
  * @param i Entier correspondant &agrave; l'index de la cellule.
  * @param e Booléen permettant l'utilisation ou non des conditions periodiques
  * @param rayon Entier correspondant au rayon de recherche des voisins.
  */
  
  public void rechercheVoisins(int i, boolean e, int rayon)
  {
    this.rechercheVoisins(i/ncol,i%ncol,e,rayon);
  }

  /**
  * Ajoute les voisins d'une cellule, dont les coordonn&eacute;es sont sp&eacute;cifi&eacute;es, pr&eacute;sents dans un certain rayon.
  * @param i Entier correspondant &agrave; l'une des coordon&eacute;es de la cellule.
  * @param j Entier correspondant &agrave; l'une des coordon&eacute;es de la cellule.
  * @param e Booléen permettant l'utilisation ou non des conditions periodiques
  * @param rayon Entier correspondant au rayon de recherche des voisins.
  */

  public void rechercheVoisins(int i,int j,boolean e,int rayon)
  {
    i=i%nlig;
    j=j%ncol;

    Cellule c = (listeCellules.get(i*ncol+j));
    int rHexa=(int)(2.0*rayon/Math.sqrt(3))+1;
    for (int ii=i-rHexa;ii<i+rHexa+1;ii++)
    {
      for (int jj=j-rayon;jj<j+rayon+1;jj++)
      {
        if (ii != i || jj != j)
        {
          Cellule cTemp = (listeCellules.get(((ii+nlig)%nlig)*ncol+((jj+ncol)%ncol)));
          double d;
          if (e)
          {
            d = this.distance(c,cTemp);
          }
          else
          {
            d = c.distance(cTemp);
          }
          if (d<=rayon+10e-6)
          {
           c.ajouterVoisin(cTemp);
          }
        }
      }
    }

    c.trim();
  }
}

