// Classe GrilleVoronoi
// Creation: 14/10/2012
// Last modification: 14/11/2012

package maillage;

import java.util.Vector;
import java.awt.geom.Point2D;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.image.*;
import java.awt.Color;
import java.awt.Polygon;

public class GrilleVoronoi extends Grille
{

  

  public GrilleVoronoi(int taille, int npoints)
  {
    //Point proot = new Point(0,0);
    //root = new Cell(proot,"sphere");
    //double probroot = 1.0/((double)(numbPoint));

    listeCellules = new Vector();

    // Generation of the initial tetrahedron
    Point pt1 = new Point(0,0);
    Point pt2 = new Point(2*taille,0);
    Point pt3 = new Point(0,2*taille);
    Point pt4 = new Point(0.5*taille,0.5*taille);

    Triangle trt1 = new Triangle(pt1, pt2, pt3);
    Triangle trt2 = new Triangle(pt1, pt3, pt4);
    Triangle trt3 = new Triangle(pt1, pt2, pt4);
    Triangle trt4 = new Triangle(pt2, pt3, pt4);

    Vector<Triangle> list_trt = new Vector<Triangle>();
    list_trt.add(trt1);
    list_trt.add(trt2);
    list_trt.add(trt3);
    list_trt.add(trt4);

    // Generation of the initial location structure
    LocationStructure graph = new LocationStructure(list_trt,taille);

    // Construction of the location structure
    graph.build((npoints-4));

    listeCellules = Cell_voronoi.cellGeneration(graph);
  }


  public int indexOf(Cellule c)
  {
    return listeCellules.indexOf(c);
  }
  
  /**
  * Effectue la conversion d'une grille en tableau de r&eacute;els en double pr&eacute;cision.
  * Il est conseill&eacute; d'utiliser cette m&eacute;thode uniquement dans le cas o&ugrave; g&eacute;n&eacute;rer 
  * un nouveau tableau est n&eacute;cessaire.
  * Attention&nbsp;! les dimensions du tableau doivent &ecirc;tre sup&eacute;rieures ou &eacute;gales &agrave; celles de la grille multipli&eacute;es par le zoom.
  * @param n Entier correspondant au nombre de lignes du tableau de sortie.
  * @param m Entier correspondant au nombre de colonnes du tableau de sortie.
  * @param gamma Entier permettant d'effectuer un zoom.
  * @return Un tableau de r&eacute;els en double pr&eacute;cision correspondant &agrave; la grille.
  */
  
  public double[][] conversion(int n,int m,int gamma)
  {
   double[][] image = new double[n][m];
   return image;
  }

  /**
  * Effectue la conversion d'une grille en tableau de r&eacute;els en double pr&eacute;cision.
  * Attention&nbsp;! les dimensions du tableau doivent &ecirc;tre sup&eacute;rieures ou &eacute;gales &agrave; celles de la grille multipli&eacute;es par le zoom.
  * @param image Un tableau de r&eacute;els en double pr&eacute;cision correspondant &agrave; la grille.
  * @param gamma Entier permettant d'effectuer un zoom.
  */
  
  public void conversion(double[][] image,int gamma)
  {
  }

  /**
  * Effectue la conversion d'une grille en un objet graphique.
  * @param graph Objet graphique correspondant &agrave; la grille.
  * @param gamma Entier permettant d'effectuer un zoom.
  */
  
  public void conversionGraphics(Graphics graph,int gamma,Color c1,Color c2)
  {
    //graph.setColor(Color.red);
    //graph.fillRect(0,0,(int)(Math.sqrt(3.0)/2.0*nlig*gamma),ncol*gamma);

    for (int k=0;k<listeCellules.size();k++)
    {
      Polygon pol = new Polygon();
      Cellule cTemp = listeCellules.get(k);
      Vector<Point> vPoint = ((Cell_voronoi)cTemp).getVoronoi();
      //System.out.println(vPoint.size());
      //System.out.println("-----");
      for (int i=0;i<vPoint.size() ;i++)
      {
        Point pTemp = vPoint.get(i);
        //System.out.println(pTemp.getX()+","+pTemp.getY());
        pol.addPoint((int)(gamma*pTemp.getX()),(int)(gamma*pTemp.getY()));
      }
      if (!cTemp.statut())
      {
        graph.setColor(c1);
      }
      else
      {
        graph.setColor(c2);
      }

      graph.fillPolygon(pol);
      //System.out.println("--o-o--");
    }

  }
  
  /**
  * Ajoute les voisins d'une cellule, dont les coordonn&eacute;es sont sp&eacute;cifi&eacute;es, pr&eacute;sents dans un certain rayon.
  * @param i Entier correspondant &agrave; l'une des coordon&eacute;es de la cellule.
  * @param j Entier correspondant &agrave; l'une des coordon&eacute;es de la cellule.
  * @param rayon Entier correspondant au rayon de recherche des voisins.
  */
  
  public void rechercheVoisins(int i,int j,int rayon)
  {
  }
  
  /**
  * Ajoute les voisins d'une cellule, dont l'index est sp&eacute;cifi&eacute;, pr&eacute;sents dans un certain rayon.
  * @param i Entier correspondant &agrave; l'index de la cellule.
  * @param rayon Entier correspondant au rayon de recherche des voisins.
  */
  
  public void rechercheVoisins(int i,int rayon)
  {
  }

    /**
  * Ajoute les voisins d'une cellule, dont l'index est sp&eacute;cifi&eacute;, pr&eacute;sents dans un certain rayon.
  * @param i Entier correspondant &agrave; l'index de la cellule.
  * @param rayon Entier correspondant au rayon de recherche des voisins.
  */
  
  public void rechercheVoisins(int i, boolean e, int rayon)
  {
  }
  
  /**
  * Retourne la plus petite distance euclidienne entre deux cellules.
  * @param c1 Premi&egrave;re cellule. 
  * @param c2 Deuxi&egrave;me cellule.
  * @return Distance entre les deux cellules.
  */

  public double distance(Cellule c1,Cellule c2)
  {
    Point q1 = c1.getPoint();
    Point q2 = c2.getPoint();
    double d = q1.distance(q2);
    double x1=q1.getX();
    double y1=q1.getY();
    double x2=q2.getX();
    double y2=q2.getY();

    for (int i=-1;i<2;i++)
    {
      for (int j=-1;j<2;j++)
      {
        d=Math.min(d,q1.distance(x2+i*nlig*Math.sqrt(3.0)*0.5,y2+j*ncol));
      }
    }

    return d;
  }
}


