// Classe Point
// Creation: 27/11/2012
// Last modification: 27/11/2012

package maillage;
import java.util.Vector;

/**
 * Classe Point
 */

public class Point
{
  private double x;
  private double y;

  /**
  * Constructeur d'un point de coordonées (0,0)
  */

  public Point ()
  {
    x=0.0;
    y=0.0;
  }

  /**
  * Constructeur d'un point à partir de deux entiers
  * @param i Position du point sur l'axe des x
  * @param j Position du point sur l'axe des y
  */

  public Point (int i,int j)
  {
    x=i;
    y=j;
  }

  /**
  * Constructeur d'un point à partir de deux réels
  * @param i Position du point sur l'axe des x
  * @param j Position du point sur l'axe des y
  */

  public Point (double i,double j)
  {
    x=i;
    y=j;
  }

  /**
  * Retourne la position du point sur l'axe des x
  * @return Position du point sur l'axe des x
  */

  public double getX ()
  {
    return x;
  }

  /**
  * Retourne la position du point sur l'axe des y
  * @return Position du point sur l'axe des y
  */

  public double getY ()
  {
    return y;
  }

  /**
  * Calcul la distance euclidienne entre deux points
  * @param i Position du point sur l'axe des x
  * @param j Position du point sur l'axe des y
  */

  public double distance (double i,double j)
  {
    return Math.sqrt((i-x)*(i-x)+(j-y)*(j-y));
  }

  /**
  * Calcul la distance euclidienne entre deux points
  * @param pts Point
  */

  public double distance (Point pts)
  {
    double i = pts.getX();
    double j = pts.getY();
    return this.distance(i,j);
  }

   /** 
    * Donne le milieu entre deux points
    * @param Q le second point
    * @return Milieu
    */

   public Point milieu(Point Q)
   {
     double xp = this.getX();
     double yp = this.getY();

     double xq = Q.getX();
     double yq = Q.getY();

     double a = (xp + xq) / 2.0;
     double b = (yp + yq) / 2.0;
  
     Point M = new Point(a,b);
     return M;
   }

  /** 
   * Donne l'intersection de deux bissectrices définit par deux points (Q et P) et un sommet S
   * @param Q Le second point qui donnera le segment SQ   
   * @param R Le troisième qui donnera le segment SR
   * @return Le point d'intersection
   */

   public Point intersec(Point Q, Point R)
   {
     // bissectrices existant dans le triangle (this,Q,R)
     Point Z1 = this.bissector(Q);
     Point Z2 = this.bissector(R);
    
     // on calcul l'intersection entre Z1 et Z2
     double Z1x= Z1.getX();
     double Z2x= Z2.getX();
     double Z1y= Z1.getY();
     double Z2y= Z2.getY();

     double x=(  Z2y - Z1y  )/( Z2x-Z1x );
     double y=Z1x*x+Z1y;
     Point I = new Point(x,y);
       
     return I;  
   }

   /** 
    * Donne les paramètres de l'équation de la droite perpendilaire à la droite constitué par deux points donnés
    * @param Q Le second point
    * @return Le vecteur contenant les paramètres a et b de la droite d'équation ax+b=y
    */

   public Point bissector(Point Q)
   {
     // On récupère les coefficients de la droite
     double x1=this.getX();
     double y1=this.getY();
     double x2=this.getX();
     double y2=this.getY();
     double a=(y2-y1)/(x2-x1);
     double b=y1-a*x1;

     // On en déduit le coefficient directeur de la droite perpendiculaire
     double c = -1*a;
     // On en déduit alors l'équation de la droite passant par le milieu de OQ
     Point M = this.milieu(Q);
     double Mx = M.getX();
     double My = M.getY();
     double d = My-c*Mx; 
   
     // Paramètres de la bissectrice R  
     Point R = new Point(c,d);

     return R;
   }




}