// Triangle class
// Creation : 21/11/2008
// Last modification : 20/01/2009

package maillage;
import java.util.Vector;
/** 
 * Class representing a triangle
 */
public class Triangle {

  private Point S1;
  private Point S2;
  private Point S3;

  private Vector<Point> vertexList;

  /**
   * Constructor of a triangle definied by 3 sommets
   * @param P Sommet #1 
   * @param Q Sommet #2
   * @param R Sommet #3
   */
  public Triangle (Point P, Point Q, Point R)
  {
    S1 = P;
    S2 = Q;
    S3 = R;

    vertexList = new Vector<Point>();
    vertexList.add(S1);
    vertexList.add(S2);
    vertexList.add(S3);
  }

  /** 
   * Returns the #1 peak of the triangle
   * @return S1
   */
  public Point getS1()
  {
    return S1;
  }

  /** 
   * Returns the #2 peak of the triangle
   * @return S2
   */
  public Point getS2()
  {
    return S2;
  }

  /** 
   * Returns the #3 peak of the triangle
   * @return S3
   */
  public Point getS3()
  {
    return S3;
  }

  /** 
   * Returns the vextex list of the triangle
   * @return Vertex list
   */
  public Vector<Point> getVertex()
  {
    return vertexList;
  }

  /**
   * Checks if the point specified as argument is contained in the triangle
   * @param P the point to be checked
   * @return answer
   */
  public boolean contains(Point P)
  {

    Vect ab= new Vect(S1, S2);
    Vect bc= new Vect(S2, S3);
    Vect ca= new Vect(S3, S1);
    
    Vect ap = new Vect(S1,P);
    Vect bp = new Vect(S2,P);
    Vect cp = new Vect(S3,P);

    if ( (ap.produitMixte(ab,ca)<=0.0) && 
	       (bp.produitMixte(ab,bc)<=0.0) && 
	       (cp.produitMixte(ca,bc)<=0.0) )
    {
      return true;
    }
    else
    {
      return false;
    }
  }
}
