// Vect class
// Creation : 18/11/2008
// Last modification : 20/01/2009
package maillage;
import java.util.Vector;
/** 
 * Class representing a vector (geometric)
 */
public class Vect {

    private Point P1;
    private Point P2;

    /**
     * Constructor of a vector PQ defined by 2 points
     * @param P Point #1 
     * @param Q Point #2
     */
    public Vect (Point P, Point Q) 
    {
     P1 = P;
     P2 = Q;
 }

    /** 
     * Returns the #1 point of the vector
     * @return P1
     */
    public Point getP1()
    {
     return P1;
 }

    /** 
     * Returns the #2 point of the vector
     * @return P2
     */
    public Point getP2()
    {
     return P2;
 }

    /** 
     * Returns the cartesian coordinates of the vector
     * @return Vector (X,Y
     */
    public Point getCoordinates()
    {
     Point A = this.getP1();
     Point B = this.getP2();


     double x2 = B.getX();
     double x1 = A.getX();
     double x = x2 - x1;

     double y2 = B.getY();
     double y1 = A.getY();
     double y = y2 - y1;

     Point pos = new Point(x,y);
     return pos;
 }

    /** 
     * Returns the norm of the vector
     * @return Norm of the vector
     */
    public double norm()
    {
     return Math.sqrt(this.scalar(this));
 }

    /** 
     * Returns the sum of 2 vectors
     * @param V Vector to add
     * @return Sum of the 2 vectors
     */

    public Vect sum(Vect V) {
     Point A = this.getCoordinates();
     Point B = V.getCoordinates();

     double x2 = B.getX();
     double x1 = A.getX();

     double y2 = B.getY();
     double y1 = A.getY();

     double x = x1 + x2;
     double y = y1 + y2;

     Point P = new Point(x,y);
     Point O = new Point(0,0);

     Vect result = new Vect(O,P);
     return result;	
 }


    /** 
     * Returns the difference of 2 vectors
     * @param V2 Vector from which the vector is substracted
     * @return Substracted vector
     */
    public Vect diff(Vect V) {
        Point A = this.getCoordinates();
        Point B = V.getCoordinates();

       double x2 = B.getX();
       double x1 = A.getX();

       double y2 = B.getY();
       double y1 = A.getY();

       double x = x1 - x2;
       double y = y1 - y2;


       Point P = new Point(x,y);
       Point O = new Point(0,0);

       Vect result = new Vect(O,P);
       return result;	
   }


    /** 
     * Returns the scalar product with another vecetor
     * @return Scalar product
     */
    public double scalar(Vect V) {
        Point A = this.getCoordinates();
        Point B = V.getCoordinates();

       double x2 = B.getX();
       double x1 = A.getX();
       double x = x2 * x1;

       double y2 = B.getY();
       double y1 = A.getY();
       double y = y2 * y1;

       double result =  x + y;
       return result;
   }


  /**
  * Retourne (A^B).(B^C)
  * @return Retourne (A^B).(B^C)
  */

  public double produitMixte (Vect a,Vect c)
  {
    Point va = a.getCoordinates();
    Point vb = this.getCoordinates();
    Point vc = c.getCoordinates();

    return (va.getX()*vb.getY()-va.getY()*vb.getX())*(vb.getX()*vc.getY()-vb.getY()*vc.getX());
}




    /** 
     * Displays the cartesian coordinates of the vector
     */
    public void printCoordinates()
    {
       System.out.println("x= " + this.getCoordinates().getX() +  " y= " + this.getCoordinates().getY() + "\n");
   }


}



